--------------------------------------------------------------------------------
-- Mapping of event handlers to event types.
--------------------------------------------------------------------------------

-- Notification display
table.insert(Event.Unit.Availability.Full, {RareNerd_show_notification_from_units, "RareNerd", "Display Notification"})

local job_coroutine = nil

local function run_job()
   -- Set yield time 10 milli sec from now
   RareNerd.next_yield = Inspect.Time.Real() + 0.01

   RareNerd_fade_notification()
   RareNerd_SetCloseMobs()
end

local function update()
   if (job_coroutine == nil) then
      job_coroutine = coroutine.create(run_job)
   end
   coroutine.resume(job_coroutine)
   if (coroutine.status(job_coroutine) == "dead") then
      job_coroutine = nil
   end
end

-- Notification fading
table.insert(Event.System.Update.Begin, {update, "RareNerd", "Fade & Close mobs"})

local raremobachvs={
   "c5C766AF68015CB70",       -- mathosia rare mobs
   "c041CBB02DEA774CE",     -- ember isle rare mobs
   "c5057BAEBDEA774CE",     -- more ember isle rare mobs
   "c128FB25EE807902B"         -- storm legion rare mobs
}

-- We have to call this function twice, once when the addon is initialized,
-- and again when we get an achievement. When Rift first starts,
-- achivement data isn't loaded yet (but we get an Event when the data
-- becomes available); when we do a /reloadui, the data is there but
-- we don't get the event. So we really have to call this from both
-- places.

local function updateachv(achv)
   local y=Inspect.Achievement.Detail(achv)
   local lang=Inspect.System.Language()
   --  print (achv .. " " .. y.name)
   for req,data in pairs(y.requirement) do
      local name=data.name
      if (name:sub(1,7) == "Besiegt") then
	 -- In the german version, ember island achievements are named
	 -- "Besiegt Tricksy" (vanquish Tricksy), so we need to remove the
	 -- "Besiegt ". This probably needs to be expanded for the english/french
	 -- versions.
	 name=name:sub(9)
      end
      --  print (req..":"..data.type.."-"..name.."-"..tostring(data.complete))

      for zone,info in pairs(RareNerd_rares[lang]) do
	 if (info[name]) then
	    info[name][6]=(data.complete or false)
	 end
      end
   end
end

-- Initialization
local function init(addon)
   if (addon == "RareNerd") then
      -- make sure the watchdog dont complain during initialization
      Command.System.Watchdog.Quiet()
      print("v1.2 loaded. We will try our best to notify you of a rare in the nearby area.")
      print("Type /rarenerd or /rn for options.")
      --      Command.System.Strict()
      RareNerd_createUI()
      local id,achv
      for id,achv in ipairs(raremobachvs) do
	 updateachv(achv)
      end
   end
end

local function gotachv(tab)
   --  print("got achv")
   for id,b in pairs(tab) do
      for i, achvid in ipairs(raremobachvs) do
	 if id == achvid then
	    updateachv(id)
	 end
      end
   end
end

table.insert(Event.Achievement.Update, { gotachv, "RareNerd", "gotachv" })
table.insert(Event.Addon.Load.End, {init, "RareNerd", "Initialization"})

local function enterSecure()
   RareNerd.secureMode = true
end

local function leaveSecure()
   RareNerd.secureMode = false
end

table.insert(Event.System.Secure.Enter, { enterSecure, "RareNerd", "EnterSecure" })
table.insert(Event.System.Secure.Leave, { leaveSecure, "RareNerd", "LeaveSecure" })

local function help()
   print("Usage: /rarenerd [option]:")
   print("    show:     show the main window")
   print("    hide:      hide the main window")
   print("    lock:       lock the main window position")
   print("    unlock:   unlock the main window position")
   print("    about:    information about RareNerd")
end

local function process(param)
   --print(param)
   if (param ~= nil) and (param ~= "") then

      local found = false

      if param == "help" then
         help()
         found = true
      end

      if param == "hide" then
         RareNerd_hideMiniWindow()
         found = true
      end

      if param == "show" then
         RareNerd_showMiniWindow()
         found = true
      end

      if param == "lock" then
         RareNerd.locked = true
         found = true
      end

      if param == "unlock" then
         RareNerd.locked = false
         found = true
      end

      if param == "about" then
         print("RareNerd was created by Mistical@Deepwood and is an updated and modified version of RareDar for Storm Legion support and additional features and bug fixes. You can check for any updates or additional Rift news and guides at http://riftnerd.com")
         found = true
      end

      if not found then
         print("Unknown option [" .. param .."] type /rarenerd help for valid options")
      end
   else
      help()
   end
end;

table.insert(Command.Slash.Register("rarenerd"), {function (params) process(params) end, "RareNerd", "Slash command"})
table.insert(Command.Slash.Register("rn"), {function (params) process(params) end, "RareNerd", "Slash command"})
