local miniWindow
local cycle, cyclepos
local lastCoordX = 0
local lastCoordZ = 0
local lastShownMessage = nil
local dbPlayerInfo = {tX,tY,tZ,
		pId = nil,
		pX = 0, pY = 0, pZ = 0,}
		
local function yieldcheck()
   if (Inspect.Time.Real() >= RareNerd.next_yield) then
      coroutine.yield()
      
      -- Set yield time 10 milli sec from now
      RareNerd.next_yield = Inspect.Time.Real() + 0.01
   end
end

-- A array for the cycle data.
-- Format of the array is:
-- [1] xccord
-- [2] ycoord
-- [3] index
-- [4] zone
-- [5] rare name
-- [6] achiev
local function updatecycleinfo(i)
   miniWindow.cycle.xpos:SetText(tostring(cycle[i][1]))
   miniWindow.cycle.ypos:SetText(tostring(cycle[i][2]))
   miniWindow.cycle.mobName:SetText(cycle[i][5])
   miniWindow.cycle.mobArea:SetText(cycle[i][4])
   miniWindow.cycle.mobName.Event.LeftDown="/target "..cycle[i][5]
   if (cycle[i][6] == true) then
      miniWindow.cycle.mobName:SetFontColor(0, 1, 0)
   elseif (cycle[i][6] == false) then
      miniWindow.cycle.mobName:SetFontColor(1, 0.5, 0.5)
   else
      miniWindow.cycle.mobName:SetFontColor(1, 1, 1)
   end
end

local function showZoneMenu()
   if RareNerd.secureMode then
      return
   end
   for i, label in ipairs(miniWindow.zoneMenu) do
      label:SetVisible(true)
   end
   miniWindow.cycle:SetVisible(false)
end

local function hideZoneMenu()
   if RareNerd.secureMode then
      return
   end
   for i, label in ipairs(miniWindow.zoneMenu) do
      label:SetVisible(false)
   end
end

function RareNerd_showMiniWindow()
   if RareNerd.secureMode then
      print("disabled during combat.")
      return
   end
   RareNerd.show = true
   miniWindow:SetVisible(true)
   print("showing main window.")
end

function RareNerd_hideMiniWindow()
   if RareNerd.secureMode then
      print("disabled during combat.")
      return
   end
   RareNerd.show = false
   miniWindow:SetVisible(false)
   print("hiding main window.")
end

-- fill mobs to the cycle array in 2 passes. First pass, add all mobs
-- that have index numbers, 2nd pass, add those that don't. So we can
-- use indexes where we have them defined, but we don't need to define
-- them in all zones.

local function zoneMenuClick(zone)
   if RareNerd.secureMode then return end
   hideZoneMenu()
   local lang=Inspect.System.Language()
   cycle={}
   for name,info in pairs(RareNerd_rares[lang][zone]) do
      if (info ~= false) and info[3] then
	 local tmpinfo=info
	 tmpinfo[4]=zone
	 tmpinfo[5]=name
	 cycle[tmpinfo[3]]=tmpinfo
      end
   end
   for name,info in pairs(RareNerd_rares[lang][zone]) do
      if (info ~= false) and (not info[3]) then
	 local tmpinfo=info
	 tmpinfo[4]=zone
	 tmpinfo[5]=name
	 table.insert(cycle, tmpinfo)
      end
   end
   cyclepos=1
   updatecycleinfo(cyclepos)
   miniWindow.cycle:SetVisible(true)
end

local function cycleLeft()
   if RareNerd.secureMode then return end
   if cyclepos==1 then cyclepos=#cycle else cyclepos=cyclepos-1 end
   updatecycleinfo(cyclepos)
end

local function cycleRight()
   if RareNerd.secureMode then return end
   if cyclepos==#cycle then cyclepos=1 else cyclepos=cyclepos+1 end
   updatecycleinfo(cyclepos)
end

local function buildMiniWindow()
   miniWindow=UI.CreateFrame("Frame", "RareNerd", context)
   miniWindow:SetPoint("TOPLEFT", UIParent, "TOPLEFT", RareNerd.xpos, RareNerd.ypos)
   miniWindow:SetWidth(250)
   miniWindow:SetHeight(50)
   miniWindow:SetBackgroundColor(0.1, 0.1, 0.1, 0.8)
   miniWindow:SetVisible(RareNerd.show)
   miniWindow:SetSecureMode("restricted")
   miniWindow.state={}
   function miniWindow.Event:LeftDown()
      if RareNerd.secureMode then return end
      miniWindow.state.mouseDown = true
      local mouse = Inspect.Mouse()
      miniWindow.state.startX = miniWindow:GetLeft()
      miniWindow.state.startY = miniWindow:GetTop()
      miniWindow.state.mouseStartX = mouse.x
      miniWindow.state.mouseStartY = mouse.y
      miniWindow:SetBackgroundColor(0.4, 0.4, 0.4, 0.8)
   end

   function miniWindow.Event:MouseMove()
      if (miniWindow.state.mouseDown and (not RareNerd.locked)) then
	 local mouse = Inspect.Mouse()
	 RareNerd.xpos = mouse.x - miniWindow.state.mouseStartX + miniWindow.state.startX
	 RareNerd.ypos = mouse.y - miniWindow.state.mouseStartY + miniWindow.state.startY
	 miniWindow:SetPoint("TOPLEFT", UIParent, "TOPLEFT",
			     RareNerd.xpos, RareNerd.ypos)
      end
   end

   function miniWindow.Event:LeftUp()
      if miniWindow.state.mouseDown then
	 miniWindow.state.mouseDown = false
	 miniWindow:SetBackgroundColor(0.1, 0.1, 0.1, 0.8)
      end
   end

   function miniWindow.Event:RightClick()
      if (miniWindow.zoneMenu[1]:GetVisible() == true) then
	 hideZoneMenu()
      else
	 showZoneMenu()
      end
   end

   miniWindow.title = UI.CreateFrame("Text", "title", miniWindow)
   miniWindow.title:SetText("RareNerd")
   miniWindow.title:SetPoint("CENTER", miniWindow, "TOPLEFT", 155, 25)
   miniWindow.title:SetFontSize(18)
   miniWindow.title:SetWidth(200);

   miniWindow.itembtn = UI.CreateFrame("Texture", "itembtn", miniWindow)
   miniWindow.itembtn:SetPoint("TOPLEFT", miniWindow, "TOPLEFT", 2, 0)
   miniWindow.itembtn:SetWidth(50)
   miniWindow.itembtn:SetHeight(50)
   miniWindow.itembtn:SetTexture("RareNerd", "RareNerd_Nothing.png")
   miniWindow.itembtn:SetSecureMode("restricted")
   miniWindow.itembtn.Event.RightClick = miniWindow.Event.RightClick

   miniWindow.zoneMenu = {}

   local lang = Inspect.System.Language()
   local zoneNames = {}
   local zoneFound = true
   for zonename,info in pairs(RareNerd_rares[lang]) do
      table.insert(zoneNames, zonename)
   end
   table.sort(zoneNames);
   if (next(zoneNames) == nil) then
      table.insert(zoneNames, "Missing zone names in ")
      table.insert(zoneNames, lang.." localization")
      zoneFound = false
   end
   for i,name in ipairs(zoneNames) do
      miniWindow.zoneMenu[i]=UI.CreateFrame("Text", "menu"..i, miniWindow)
      miniWindow.zoneMenu[i]:SetText(name)
      miniWindow.zoneMenu[i]:SetFontSize(15)
      miniWindow.zoneMenu[i]:SetWidth(250)
      miniWindow.zoneMenu[i]:SetVisible(false)
	  miniWindow.zoneMenu[i]:SetFontColor(0, 0, 0, 1)
      miniWindow.zoneMenu[i]:SetBackgroundColor(0.50,0.50,0.50, 1)
      if (i==1) then
	 miniWindow.zoneMenu[i]:SetPoint("TOPLEFT", miniWindow, "BOTTOMLEFT", 0, 0)
      else
	 miniWindow.zoneMenu[i]:SetPoint("TOPLEFT", miniWindow.zoneMenu[i-1], "BOTTOMLEFT", 0, 0)
      end
      if zoneFound then
	 miniWindow.zoneMenu[i].Event.LeftClick=function()
	    zoneMenuClick(name)
	 end
	 miniWindow.zoneMenu[i].Event.MouseIn=function()
	    miniWindow.zoneMenu[i]:SetBackgroundColor(0.12,0.56,1.00, 1)
	 end
	 miniWindow.zoneMenu[i].Event.MouseOut=function()
	    miniWindow.zoneMenu[i]:SetBackgroundColor(0.50,0.50,0.50, 1)
	 end
      end
   end

   miniWindow.cycle=UI.CreateFrame("Frame", "Cycle", miniWindow)
   miniWindow.cycle:SetWidth(250)
   miniWindow.cycle:SetHeight(62)
   miniWindow.cycle:SetVisible(false)
   miniWindow.cycle:SetBackgroundColor(0.1, 0.1, 0.1, 0.8)
   miniWindow.cycle:SetPoint("TOPLEFT", miniWindow, "BOTTOMLEFT", 0, 0)
   miniWindow.cycle:SetSecureMode("restricted")

   miniWindow.cycle.mobName=UI.CreateFrame("Text", "CycleMobName", miniWindow.cycle)
   miniWindow.cycle.mobName:SetWidth(180)
   miniWindow.cycle.mobName:SetHeight(20)
   miniWindow.cycle.mobName:SetFontSize(14)
   miniWindow.cycle.mobName:SetPoint("TOPLEFT", miniWindow.cycle, "TOPLEFT", 35, 0)
   miniWindow.cycle.mobName:SetSecureMode("restricted")

   miniWindow.cycle.mobArea=UI.CreateFrame("Text", "CycleMobName", miniWindow.cycle)
   miniWindow.cycle.mobArea:SetWidth(140)
   miniWindow.cycle.mobArea:SetHeight(20)
   miniWindow.cycle.mobArea:SetFontSize(14)
   miniWindow.cycle.mobArea:SetPoint("TOPLEFT", miniWindow.cycle.mobName, "BOTTOMLEFT", 0, 0)

   miniWindow.cycle.leftshift=UI.CreateFrame("Texture", "CycleLeft", miniWindow.cycle)
   miniWindow.cycle.leftshift:SetWidth(35)
   miniWindow.cycle.leftshift:SetHeight(50)
   miniWindow.cycle.leftshift:SetTexture("RareNerd", "arrow_left.png")
   miniWindow.cycle.leftshift:SetPoint("TOPLEFT", miniWindow.cycle, "TOPLEFT", 0, 8)
   miniWindow.cycle.leftshift.Event.LeftClick=cycleLeft

   miniWindow.cycle.rightshift=UI.CreateFrame("Texture", "CycleRight", miniWindow.cycle)
   miniWindow.cycle.rightshift:SetWidth(35)
   miniWindow.cycle.rightshift:SetHeight(50)
   miniWindow.cycle.rightshift:SetTexture("RareNerd", "arrow_right.png")
   miniWindow.cycle.rightshift:SetPoint("TOPRIGHT", miniWindow.cycle, "TOPRIGHT", 0, 8)
   miniWindow.cycle.rightshift.Event.LeftClick=cycleRight

   miniWindow.cycle.xpos=UI.CreateFrame("Text", "CycleMobXPos", miniWindow.cycle)
   miniWindow.cycle.xpos:SetWidth(60)
   miniWindow.cycle.xpos:SetFontSize(14)
   miniWindow.cycle.xpos:SetPoint("TOPLEFT", miniWindow.cycle.mobArea, "BOTTOMLEFT", 0, 0)

   miniWindow.cycle.ypos=UI.CreateFrame("Text", "CycleMobYPos", miniWindow.cycle)
   miniWindow.cycle.ypos:SetWidth(60)
   miniWindow.cycle.ypos:SetFontSize(14)
   miniWindow.cycle.ypos:SetPoint("TOPRIGHT", miniWindow.cycle.mobArea, "BOTTOMRIGHT", -20, 0)
end

function RareNerd_SetZoneMobs(list)
   local str = ""
   local message = ""
   local n = 0
   local max = table.maxn(list)
   for i,name in ipairs(list) do
      yieldcheck()
      n = i
      str = str .. "target " .. name .. "\n"
      if (n > 1) then
	 if (n == max) then
	    message = message .. " and "
	 else
	    message = message .. ", "
	 end
      end
      message = message .. name
   end

   yieldcheck()

   if (n > 0) then
      if (lastShownMessage ~= message) then
	 print(message .. " might be close")
      end
      miniWindow.itembtn:SetTexture("RareNerd", "RareNerd_Searching.png")
   else
      miniWindow.itembtn:SetTexture("RareNerd", "RareNerd_Nothing.png")
   end
   lastShownMessage = message;

   miniWindow.title:SetText("RareNerd (" .. n .. ")")
   miniWindow.itembtn.Event.LeftDown = str
end

local function startup()
	if (addonLoaded == RareNerd) then
		if (dbPlayerInfo.pId == nil) then 
			dbPlayerInfo.pId = Inspect.Unit.Lookup("player")
			print(dbPlayerInfo)
			createFrames()
		end
	end
end

function RareNerd_SetCloseMobs()
   if (not RareNerd.secureMode) then
      yieldcheck()
      local player = Inspect.Unit.Detail("player");
      if ((player.coordX ~= nil) and (player.coordY ~= nil) and
     ((math.abs(player.coordX - lastCoordX) >= 20) or
      (math.abs(player.coordZ - lastCoordZ) >= 20))) then
         local lang = Inspect.System.Language()
         local moblist = {}
         if (player.zone ~= nil) then
            local zone=Inspect.Zone.Detail(player.zone)
            local lang_rares = RareNerd_rares[lang]
            if (lang_rares ~= nil) then
               local zone_rares = lang_rares[zone.name]
               if (zone_rares ~= nil) then
                  for name, info in pairs(zone_rares) do
                     yieldcheck()
                     if info then
                        local xdist = math.abs(player.coordX - info[1])
                        local zdist = math.abs(player.coordZ - info[2])
                         if (xdist <= 150 and zdist <= 150) then
                           table.insert(moblist, name)
                        end
                     end
                  end
                end
            end
         end

         RareNerd_SetZoneMobs(moblist)
         lastCoordX = player.coordX;
         lastCoordZ = player.coordZ;
      end
   end
end

function RareNerd_createUI()
   context = UI.CreateContext("RareNerd")
   context:SetSecureMode("restricted")

   if (miniWindow == nil) then
      buildMiniWindow()
   end
end
